def build_request(author, payload=None, display=None):
    """
    """
    request = {
        'type':
        'request',
        'from':
        author,
        'tts':
        payload if payload else
        (author + ' is waiting for your answer in order to proceed.'),
        'display':
        display
    }
    return request



import hashlib

def hash_plugin(path):
    BUF_SIZE = 65536
    hasher = hashlib.sha256()
    with open(path, 'rb') as f:
        while True:
            data = f.read(BUF_SIZE)
            if not data:
                break
            hasher.update(data)
    return hasher.hexdigest()



import json
import os
import shutil
import zipfile

from ...exceptions import RuntimeError


def split_string(string_to_split, separator):
    """
    """
    result = string_to_split.split(separator)
    if len(result) == 1:
        return result[0], None
    return result[0], ' '.join(result[1:])


def remove_file(path):
    """
    Removes the given file if it exists, raise an error otherwise.
        @param:
            - path: string (/path/to/the/file/to/remove)

        @behave: raise an error if the specified file does not exist.
    """
    if os.path.isfile(path) == True:
        os.remove(path)
    else:
        raise RuntimeError(__name__, remove_file.__name__,
                           "Cannot remove (" + path + "), no such file.")


def remove_directory(path):
    """
    Handler for removing a directory and all its content
        @param:
            - path: string (/path/to/the/directory/to/remove)

        @behave: raise an error if the specified directory does not exist.
    """
    if os.path.isdir(path) == True:
        shutil.rmtree(path)
    else:
        raise RuntimeError(__name__, remove_directory.__name__,
                           "Cannot remove (" + path + "), no such directory.")


def unzip(path, destination):
    """
    Unzip the file pointed by 'path' to extract its to content to the given destination
        @params:
            - path: string (/path/to/the/file/to/unzip)
            - destination: string (/path/to/extract/the/zip)

        @behave: raise an error if either the path or the destination is invalid.
        @behave: raise an error if the archive is corrupted.
    """
    if os.path.isfile(path) == True:

        if os.path.isdir(destination) == True:
            archive = zipfile.ZipFile(path, 'r')
            result = archive.testzip()

            if result is not None:
                raise RuntimeError(__name__, unzip.__name__,
                                   "Error corrupted archive.")

            archive.extractall(destination)
            archive.close()

        else:
            raise RuntimeError(
                __name__, unzip.__name__,
                "Invalid path (" + destination + "), no such directory.")
    else:
        raise RuntimeError(__name__, unzip.__name__,
                           "Invalid path (" + path + "), no such file.")


def load_manifest_to_dictionary(path, dictionary):
    """
    Loads the manifest.json of the plugin folder specified by 'path' to the given
    dictionary.

        @params:
            - path: string (/path/to/the/folder/containing/the/json/file)
            - dictionary: the dictionary to fill with the data.

        @behave: raise an error if the path is not pointing to a directory.
    """
    if os.path.isdir(path):
        for file in os.listdir(path):
            if file.find(".json") > 0:
                with open(os.path.join(path, file)) as json_file:
                    data = json.load(json_file)
                for key, value in data.items():
                    dictionary[key] = value
    else:
        raise RuntimeError(__name__, parse_json_file_to_dictionary.__name__,
                           "Invalid path (" + path + "), no such directory.")


def determine_language(path, name, dictionary, skip):
    """
    Determines in which language the plugin located at 'path' is written, and adds
    its 'name' as well as the language to the given 'dictionary'.

    For example, if the specified folder contains a plugin named 'hello_world' written
    in C++, the dictionary will look like following:

        {'hello_world': {'lang': 'cpp'}}

    @param:
        - dictionary: dictionary (the dictionary to fill)
        - path: string (path to the plugin folder)
        - name: string (the plugin name)
        - skip: array of strings  (extension to skip i.e 'json')

    """
    if os.path.isdir(path):
        for file in os.listdir(path):
            if file == 'setup.py':
                continue
            if file.find(".") > 0 and file[file.find(".") + 1:] not in skip:
                dictionary[name] = {'lang': file[file.find(".") + 1:]}


def load_plugin(path, name):
    """
    Returns a well formed dictionary built from the manifest.json found in the plugin
    folder located at 'path/name'.
    """
    dictionary = {}
    target = os.path.join(path, name)
    determine_language(target, name, dictionary, ['json', 'md', 'txt'])
    load_manifest_to_dictionary(target, dictionary[name])
    return dictionary



def build_response(author, result=None, display=None):
    """
    """
    response = {
        'type': 'response',
        'from': author,
        'tts': result if result else (author + ' just finished to proceed.'),
        'display': display
    }
    return response



def popup_macos(title, content):
    # import os
    # script = 'osascript -e '
    # args = "'display notification \"{}\" with title \"{}\"'".format(content, title)
    # os.system(script + args)
    print(content)

def popup(title, content):
    import tkinter
    import tkinter.scrolledtext
    win = tkinter.Tk()
    win.title(title)
    frame = tkinter.Frame(master=win, bg='#808000')
    frame.pack(fill='both', expand='yes')
    editArea = tkinter.scrolledtext.ScrolledText(
        master=frame, wrap=tkinter.WORD, width=80, height=40)
    editArea.pack(padx=10, pady=10, fill=tkinter.BOTH, expand=True)
    editArea.insert(tkinter.INSERT, content)
    editArea.configure(state="disabled")
    win.mainloop()


class Logger(object):
    """
    """
    ERROR = '__AVA__PLUGIN__ERROR__'
    IMPORT = '__AVA__PLUGIN__IMPORT__'
    REQUEST = '__AVA__PLUGIN__REQUEST__'
    RESPONSE = '__AVA__PLUGIN__RESPONSE__'
    DELIMITER = '__AVA__PLUGIN__DELIMITER__'

    @staticmethod
    def log_error():
        print(Logger.ERROR)
        print(Logger.DELIMITER)

    @staticmethod
    def log_import():
        print(Logger.IMPORT)
        print(Logger.DELIMITER)

    @staticmethod
    def log_request():
        print(Logger.REQUEST)
        print(Logger.DELIMITER)

    @staticmethod
    def log_response():
        print(Logger.RESPONSE)
        print(Logger.DELIMITER)

    @staticmethod
    def unexpected_error(self):
        return 'An unexpected error occured with the {0}. The error has been print.'.format(
            self.__class__.__name__)

    @staticmethod
    def popup(title, content):
        import platform
        if platform.system() == 'Darwin':
            import threading
            t = threading.Thread(target=popup_macos, args=(title, content))
            t.daemon = True
            t.start()
            return
        import multiprocessing
        p = multiprocessing.Process(
            name='AVA.popup.{0}'.format(title),
            target=popup,
            args=(title, content))
        p.daemon = True
        p.start()



def build_event(raw):
    """
    """
    index = 0
    results = ' '.join(raw.split()).split(' ')
    event = {'action': None, 'target': None, 'args': None}
    for key, result in zip(event, results):
        event[key] = ' '.join(
            map(str, results[index:])) if key == 'args' else result
        index += 1
    return event



from setuptools import setup, find_packages

import avasdk

keywords = [
    'ava', 'sdk',
]

setup(
    name='avasdk',
    version=avasdk.__version__,
    description=avasdk.__doc__,
    author=avasdk.__author__,
    license='Apache',
    url=avasdk.__url__,
    keywords=keywords,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    install_requires=['valideer'],
)



import valideer
from valideer.base import ValidationError as ValideerValidationError
from ...exceptions import ValidationError

schema = {
    "+name": "string",
    "+description": "string",
    "+version": "string",
    "+author": "string",
    "+source": "string",
    "+commands": [
        {
            "+name": "string",
            "+description": "string"
        }
    ],
    "tags": ["string"],
    "build": "boolean"
}

validator = valideer.parse(schema)

def validate_manifest(manifest):
    try:
        return validator.validate(manifest)
    except ValideerValidationError as e:
        raise ValidationError(e)



from ..log import Logger
from ..builders import build_request


class PythonModel(object):
    """
        Base Class of AVA python plugins
    """

    def __init__(self, name="AVA_Python_Plugin_Model"):
        super(PythonModel, self).__init__()
        PythonModel._commands = {
            "name": self.get_name,
            "list": self.list_commands,
            "commands": self.get_commands,
            "interaction": self.wait_for_user
        }
        PythonModel._name = name

    def set_commands_list(self, command_list):
        self._commands = command_list

    def get_name(self):
        return self._name

    def get_commands(self):
        return self._commands

    def list_commands(self):
        for c in self._commands:
            print(c)

    def wait_for_user(self, author, message=None, display=None):
        print(build_request(author, message, display))
        Logger.log_request()
        return input()
